#----------------------------------------------------------------------
# Copyright 1999-2003 Mitel Networks Corporation
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
#----------------------------------------------------------------------

package esmith::rpm;

require 5.006;
use strict;
use warnings;
use Carp;

our $VERSION = sprintf '%d.%03d', q$Revision: 1.19 $ =~ /: (\d+).(\d+)/;


=head1 NAME

esmith::rpm - Abstract RPM manipulation class

=head1 SYNOPSIS

  use esmith::rpm;

  my $rpm = esmith::rpm->new($rpm_filename);

  my $file    = $rpm->file;
  my $name    = $rpm->name;
  my $version = $rpm->version;
  my $release = $rpm->release;
  my $arch    = $rpm->arch;

  my($name, $version, $release, $arch) = $rpm->tags;
  my $tag = $rpm->tag;

  $rpm->reload;

  $rpm1->cmp($rpm2);

=head1 DESCRIPTION

This is an abstract class encapsulating working with individual RPMs.

It is not useful by itself, it serves to define an interface.  You
must subclass this and, where noted, define the methods yourself.

=head2 Methods

=over 4

=item I<new>

  my $rpm = esmith::rpm->new($rpm_filename);

Initializes a new esmith::rpm object representing the given
$rpm_filename, which must be a real RPM.

=for testing
use_ok('esmith::rpm');

=cut

sub new {
    my($proto, $rpm_filename) = @_;
    my($class) = ref $proto || $proto;

    carp "$rpm_filename does not exist"  unless -e $rpm_filename;
    carp "$rpm_filename is not readable" unless -r $rpm_filename;

    return bless { file => $rpm_filename }, $proto;
}

=item I<reload>

  $rpm->reload;

Rereads the RPM file.

=cut

sub reload {
    my($self) = shift;

    %$self = %{$self->new($self->file)};
    return 1;
}


=item I<file>

  my $file = $rpm->file;

Returns the RPM filename which $rpm represents

=cut

sub file { $_[0]->{file} }

=head2 Identity

An RPM's identity is broken down into:

   <name>-<version>-<release>.<arch>.rpm

=over 4

=item I<increment_version>

=cut

sub increment_version 
{
    my $self = shift;
    return _increment_alphanum($self->version);
}

=item I<increment_release>

=cut

sub increment_release
{
    my $self = shift;
    return _increment_alphanum($self->release);
}

=item I<cmp>

  my $cmp = $rpm->cmp($other_rpm);

Compares the name, version, release and arch one $rpm to the $other_rpm.
Returns -1, 0 or 1 if $rpm is <, == or > the $other_rpm.

=for _those_who_follow
Do not attempt to replace this with RPM::vercmp().  It doesn't work.

=begin testing

use_ok('esmith::rpm::filename');

my ($nameA,$nameB,$rpmA, $rpmB);
my @cmpArrays = (
    [ "bind-9.2.1-0.70.i386.rpm", 	"bind-9.2.1-07x.i386.rpm", 	-1 ],
    [ "perl-5.00503-1.i386.rpm",	"perl-5.6.1-1.i386.rpm", 	-1 ],
    [ "quota-2.00-7.i386.rpm",	"quota-2.00-7es.i386.rpm", 	-1 ],
    [ "quota-2.00-7es.i386.rpm",	"quota-2.00-7.i386.src.rpm", 	1 ],
    [ "e-smith-lib-1.9.58-01.noarch.rpm","e-smith-lib-1.9.9-01.noarch.rpm", 1 ],
    [ "ucd-snmp-4.2.5-7.73.0.i386.rpm","ucd-snmp-4.2-12cmaX.i386.rpm", 1 ],
);

foreach my $cmpArray (@cmpArrays)
{
    my ($nameA, $nameB, $res) = @$cmpArray;
    $rpmA = esmith::rpm::filename->new($nameA);
    $rpmB = esmith::rpm::filename->new($nameB);
    is($rpmA->cmp($rpmB), $res, "$nameA cmp $nameB");
}

=end testing

=cut

sub cmp
{
    my ($rpma, $rpmb) = @_;

    $rpma->name cmp $rpmb->name
      or
    _cmp_alphanum($rpma->version, $rpmb->version)
      or 
    _cmp_alphanum($rpma->release, $rpmb->release)
      or
    _cmp_alphanum($rpma->arch,    $rpmb->arch)
}   

=begin _private

=item B<_cmp_alphanum>

  my $cmp = _cmp_alphanum($a, $b);

Compare a string in alpha and numeric chunks.
Returns -1, 0 or 1 if the first argument is <, =, or > the second.
The first parameter dictates the expected type.
ie:

    cmp_alphanum("10", "b"):  a numeric type is not found for 
    the second parameter, and so 0 is used.  10 > 0.

    cmp_alphanum("b", "10"):  a non-numeric type is not found
    for the second parameter, and so the empty string is used.  "b" > ""

=end _private

=cut

sub _split_alphanum
{
    my ($alphanum) = @_;
    return () unless ($alphanum);

    my ($type, $first, $dot);

    my @parts;

    while ($alphanum ne '') {
	if ($alphanum =~ /^((?:\d+|\D+))(\.?)(.*)$/) {
	    ($first, $dot, $alphanum) = ($1, $2, $3);
	    if ($first =~ /^\d+$/) { $type = "num"; }
	    elsif ($first =~ /^\D+$/) { $type = "string"; }
	    push @parts, [$type, $first, $dot, $alphanum];
	}
    }

    return @parts;
}

=begin testing

my ($a,$r);
$a = "aa123";
is($r=esmith::rpm::_increment_alphanum($a), "aa124", "$a ++ = $r"); 
$a = "aa123aa234";
is($r=esmith::rpm::_increment_alphanum($a), "aa123aa235", "$a ++ = $r"); 
$a = "123aa";
is($r=esmith::rpm::_increment_alphanum($a), "124aa", "$a ++ = $r"); 
$a = "123.aa";
is($r=esmith::rpm::_increment_alphanum($a), "124.aa", "$a ++ = $r"); 
$a = "7es0";
is($r=esmith::rpm::_increment_alphanum($a), "7es1", "$a ++ = $r"); 
$a = "01es0";
is($r=esmith::rpm::_increment_alphanum($a), "01es1", "$a ++ = $r"); 

=end testing

=cut

sub _increment_alphanum
{
    my ($alphanum) = @_;

    my $result;
    my @parts = _split_alphanum($alphanum);
    my $old = "";
    my $new;
    for (my $i = scalar(@parts)-1; $i >= 0; $i--)
    {
	#next if ($parts[$i][0] eq "string");
	if ($parts[$i][0] eq "num") {
	    $old = $parts[$i][1];
	    my $n = length($old);
	    $new = sprintf "%0${n}d", $parts[$i][1] + 1;
	    last;
	}
    }
    if (defined $new)
    {
	$alphanum =~ s/$old(?!.*?$old)/$new/;
    }
    else
    {
	$alphanum .= "1";
    }
    return $alphanum;
}

=begin testing

use esmith::rpm;

my ($a,$b);
($a,$b) = ("aa123","aa123");
is(esmith::rpm::_cmp_alphanum($a, $b), 0, "$a cmp $b");
($a,$b) = ("aa123","aa234");
is(esmith::rpm::_cmp_alphanum($a, $b), -1, "$a cmp $b");
($a,$b) = ("aa123","bb234");
is(esmith::rpm::_cmp_alphanum($a, $b), -1, "$a cmp $b");
($a,$b) = ("bb123","aa123");
is(esmith::rpm::_cmp_alphanum($a, $b), 1, "$a cmp $b");
($a,$b) = ("bb123","aa234");
is(esmith::rpm::_cmp_alphanum($a, $b), 1, "$a cmp $b");
($a,$b) = ("bb123","aa234");
is(esmith::rpm::_cmp_alphanum($a, $b), 1, "$a cmp $b");
($a,$b) = ("1.0","1");
is(esmith::rpm::_cmp_alphanum($a, $b), 0, "$a cmp $b");
($a,$b) = ("1","1.0.0");
is(esmith::rpm::_cmp_alphanum($a, $b), 0, "$a cmp $b");

=end testing

=cut

sub _cmp_alphanum
{
    my ($a, $b) = @_;
    return 0 unless ($a || $b);
    my @partsA = _split_alphanum($a);
    my @partsB = _split_alphanum($b);
    my $result = 0;
    while (@partsA || @partsB)
    {
	my $partA = shift @partsA || ['','','',''];
	my $partB = shift @partsB || ['','','',''];
	my ($typeA, $firstA, $dotA, $restA) = @$partA;
	my ($typeB, $firstB, $dotB, $restB) = @$partB;

	if ($firstA =~ /^0*$/ && $firstB =~ /^0*$/) {
	    # special case for things like 4.2.0 cmp 4.2
	    # (4.2 cmp 4.2pre) == 1, which is what we want
	    # XXX: (4.2 cmp 4.2.pre) == 1, which might not be good
	    $result = _cmp_pre($restA, $restB);
	}
	elsif ($typeA ne "string" && $typeB ne "string")
	{
	    # left-padded zeros lose. 
	    # this does the right thing for cases like: 
	    # -> perl-5.00503 cmp perl-5.6.0 
	    # -> e-smith-lib-1.9.58 cmp e-smith-lib-1.9.9
	    my $lpadA; 
	    for ($lpadA=0; $lpadA < length($firstA); $lpadA++) {
		last unless (substr($firstA, $lpadA, 1) eq "0");
	    }
	    my $lpadB; 
	    for ($lpadB=0; $lpadB < length($firstB); $lpadB++) {
		last unless (substr($firstB, $lpadB, 1) eq "0");
	    }
	    
	    $result = ($lpadB <=> $lpadA) 
		    || (($firstA || 0) <=> ($firstB || 0))
		    || _cmp_pre($restA, $restB);
	}
	else
	{
	    $result = ("$firstA$dotA" cmp "$firstB$dotB");
	}
	last if $result != 0;
    }
    return $result;
}

=begin testing

use esmith::rpm;

my ($a,$b);
($a,$b) = ("aa123","aa123");
is(esmith::rpm::_cmp_alphanum_old($a, $b), 0, "$a cmp $b");
($a,$b) = ("aa123","aa234");
is(esmith::rpm::_cmp_alphanum_old($a, $b), -1, "$a cmp $b");
($a,$b) = ("aa123","bb234");
is(esmith::rpm::_cmp_alphanum_old($a, $b), -1, "$a cmp $b");
($a,$b) = ("bb123","aa123");
is(esmith::rpm::_cmp_alphanum_old($a, $b), 1, "$a cmp $b");
($a,$b) = ("bb123","aa234");
is(esmith::rpm::_cmp_alphanum_old($a, $b), 1, "$a cmp $b");
($a,$b) = ("bb123","aa234");
is(esmith::rpm::_cmp_alphanum_old($a, $b), 1, "$a cmp $b");

=end testing

=cut

sub _cmp_alphanum_old
{
    my ($a, $b) = @_;
    return 0 unless ($a || $b);
    my $type;
    my ($firstA, $dotA, $restA);
    my ($firstB, $dotB, $restB);

    if ($a =~ /^(\d+)(\.?)(.*)$/) {
        ($type, $firstA, $dotA, $restA) = ("num", $1, $2 || "", $3);

        if ($b =~ /^(\d*)(\.?)(.*)$/) {
            ($firstB, $dotB, $restB) = ($1 || 0, $2 || "", $3);
        }
    }
    elsif ($a =~ /^(\D+)(\.?)(.*)$/) {
        ($type, $firstA, $dotA, $restA) = ("string", $1, $2 || "", $3);

        if ($b =~ /^(\D*)(\.?)(.*)$/) {
            ($firstB, $dotB, $restB) = ($1 || "", $2 || "", $3);
        }
    }
    elsif($b =~ /^(\d+)(\.?)(.*)$/) {
        ($type, $firstB, $dotB, $restB) = ("num", $1 || 0, $2 || "", $3);

        if ($a =~ /^(\d*)(\.?)(.*)$/) {
            ($firstA, $dotA, $restA) = ($1 || 0, $2 || "", $3);
        }
    }
    elsif($b =~ /^(\D+)(\.?)(.*)$/) {
        ($type, $firstB, $dotB, $restB) = ("string", $1 || "", $2 || "", $3);

        if ($a =~ /^(\D*)(\.?)(.*)$/) {
            ($firstA, $dotA, $restA) = ($1 || "", $2 || "", $3);
        }
    }

    if ($type eq "num") {
        return (($firstA || 0) <=> ($firstB || 0)) ||
               _cmp_pre($restA, $restB) ||
               _cmp_alphanum($restA, $restB);
    }
    else {
        return ("$firstA$dotA" cmp "$firstB$dotB") || 
               _cmp_alphanum($restA, $restB);
    }
}


# special case for comparing 2.00pre4 vs 2.00
sub _cmp_pre {
    my($a, $b) = @_;

    if( $a =~ /^pre/ xor $b =~ /^pre/ ) 
    {
        return -1 if $a =~ /^pre/;
        return 1  if $b =~ /^pre/;
    }
    else 
    {
        return 0;
    }
}

=item B<name>       ABSTRACT

=item B<version>    ABSTRACT

=item B<release>    ABSTRACT

=item B<arch>       ABSTRACT

  my $name    = $rpm->name;
  my $version = $rpm->version;
  my $release = $rpm->release;
  my $arch    = $rpm->arch;

=item B<_target_cpu>

Fetch the default _target_cpu macro for the system.

=cut

our $_target_cpu;

sub _target_cpu
{
    unless (defined $_target_cpu)
    {
	chomp($_target_cpu = `rpm --eval '%{_target_cpu}'`);
    }
    return $_target_cpu;
}

=pod

Accessors to get at the RPM's tags.

=item B<tag>

  my $tag = $rpm->tag;

Returns a reconstruction of the rpm's name, version, release and arch
suitable for filenaming.

    name-version-release.arch

=cut

sub tag {
    my($self) = shift;
    return sprintf "%s-%s-%s.%s", $self->tags;
}

=item B<tags>

  my($name, $version, $release, $arch) = $rpm->tags;

Convenience accessor to get all the tags at once.

=cut

sub tags {
    my($self) = shift;
    return map { $self->$_() } qw(name version release arch);
}

=back

=head1 AUTHOR

Mitel Networks Corporation

=cut

1;
