#----------------------------------------------------------------------
# copyright (C) 2002 Mitel Networks Corporation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
#
# Technical support for this program is available from Mitel Networks
# Please visit our web site www.e-smith.com for details.
#----------------------------------------------------------------------

package esmith::rpm;

require 5.006;
use strict;
use warnings;
use Carp;

our $VERSION = sprintf '%d.%03d', q$Revision: 1.3 $ =~ /: (\d+).(\d+)/;


=head1 NAME

esmith::rpm - RPM manipulation class

=head1 SYNOPSIS

  use esmith::rpm;

  my $rpm = esmith::rpm->new($rpm_filename);

  my $file    = $rpm->file;
  my $name    = $rpm->name;     *UNIMPLEMENTED*
  my $version = $rpm->version;  *UNIMPLEMENTED*
  my $release = $rpm->release;  *UNIMPLEMENTED*
  my $arch    = $rpm->arch;     *UNIMPLEMENTED*

  esmith::rpm->cmp($rpm1, $rpm2);

=head1 DESCRIPTION

This class encapsulates working with individual RPM files.

=head2 Methods

=over 4

=item B<new>

  my $rpm = esmith::rpm->new($rpm_filename);

Initializes a new esmith::rpm object representing the given
$rpm_filename, which must be a real RPM.

=cut

sub new {
    my($proto, $rpm_filename) = @_;
    my($class) = ref $proto || $proto;

    carp "$rpm_filename does not exist"  unless -e $rpm_filename;
    carp "$rpm_filename is not readable" unless -r $rpm_filename;

    return bless { file => $rpm_filename }, $proto;
}

=item B<file>

  my $file = $rpm->file;

Returns the RPM filename which $rpm represents

=cut

sub file { $_[0]->{file} }

=head2 Identity

An RPM's identity is broken down into:

   <name>-<version>-<release>.<arch>.rpm

=over 4

=item B<cmp>

  my $cmp = $rpm->cmp($other_rpm);

Compares the name, version, release and arch one $rpm to the $other_rpm.
Returns -1, 0 or 1 if $rpm is <, == or > the $other_rpm.

=cut

sub cmp
{
    my ($rpma, $rpmb) = @_;

    $rpma->name cmp $rpmb->name
      or
    _cmp_alphanum($rpma->version, $rpmb->version)
      or 
    _cmp_alphanum($rpma->release, $rpmb->release)
      or
    _cmp_alphanum($rpma->arch,    $rpmb->arch)
}   

=begin _private

=item B<_cmp_alphanum>

  my $cmp = _cmp_alphanum($a, $b);

Compare a string in alpha and numeric chunks.
Returns -1, 0 or 1 if the first argument is <, =, or > the second.
The first parameter dictates the expected type.
ie:

    cmp_alphanum("10", "b"):  a numeric type is not found for 
    the second parameter, and so 0 is used.  10 > 0.

    cmp_alphanum("b", "10"):  a non-numeric type is not found
    for the second parameter, and so the empty string is used.  "b" > ""

=end _private

=cut

sub _cmp_alphanum
{
    my ($a, $b) = @_;
    return 0 unless ($a || $b);
    my $type;
    my ($firstA, $dotA, $restA);
    my ($firstB, $dotB, $restB);

    if ($a =~ /^([\d]+)(\.)?(.*)$/) {
        ($type, $firstA, $dotA, $restA) = ("num", $1, $2 || "", $3);

        if ($b =~ /^([\d]+)?(\.)?(.*)$/) {
            ($firstB, $dotB, $restB) = ($1 || 0, $2 || "", $3);
        }
    }
    elsif ($a =~ /^([^\d]+)(\.)?(.*)$/) {
        ($type, $firstA, $dotA, $restA) = ("string", $1, $2 || "", $3);

        if ($b =~ /^([^\d]+)?(\.)?(.*)$/) {
            ($firstB, $dotB, $restB) = ($1 || "", $2 || "", $3);
        }
    }
    elsif($b =~ /^([\d]+)(\.)?(.*)$/) {
        ($type, $firstB, $dotB, $restB) = ("num", $1 || 0, $2 || "", $3);

        if ($a =~ /^([\d]+)?(\.)?(.*)$/) {
            ($firstA, $dotA, $restA) = ($1 || 0, $2 || "", $3);
        }
    }
    elsif($b =~ /^([^\d]+)(\.)?(.*)$/) {
        ($type, $firstB, $dotB, $restB) = ("string", $1 || "", $2 || "", $3);

        if ($a =~ /^([^\d]+)?(\.)?(.*)$/) {
            ($firstA, $dotA, $restA) = ($1 || "", $2 || "", $3);
        }
    }

    if ($type eq "num") {
        return (($firstA || 0) <=> ($firstB || 0)) ||
               _cmp_pre($restA, $restB) ||
               _cmp_alphanum($restA, $restB);
    }
    else {
        return ("$firstA$dotA" cmp "$firstB$dotB") || 
               _cmp_alphanum($restA, $restB);
    }
}


# special case for comparing 2.00pre4 vs 2.00
sub _cmp_pre {
    my($a, $b) = @_;

    if( $a =~ /^pre/ xor $b =~ /^pre/ ) 
    {
        return -1 if $a =~ /^pre/;
        return 1  if $b =~ /^pre/;
    }
    else 
    {
        return 0;
    }
}

=item B<name>

  my $name = $rpm->name;

=item B<version>

  my $version = $rpm->version;

=item B<release>

  my $release = $rpm->release;

=item B<arch>

  my $arch = $rpm->arch;

=back

=head1 AUTHOR

Mitel Network Solutions, Inc.

=cut

1;
