#----------------------------------------------------------------------
# Copyright 1999-2003 Mitel Networks Corporation
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
#----------------------------------------------------------------------

package esmith::rpm::cmp;

use strict;
our $VERSION = sprintf '%d.%03d', q$Revision: 1.6 $ =~ /: (\d+).(\d+)/;

require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(sense2cmp);


=for testing
use_ok('esmith::rpm::cmp');


=head1 NAME

esmith::rpm::cmp - Magic comparison number for esmith::rpm.

=head1 SYNOPSIS

  use esmith::rpm::cmp;

  my $cmp = esmith::rpm::cmp->new($name, $version, $release, $sense);

  my $name    = $cmp->name;
  my $version = $cmp->version;
  my $release = $cmp->release;
  my $sense   = $cmp->sense;

  my $comparison_as_string = $cmp->as_string;
  my $sense_as_string      = $cmp->sense_as_string;

  $cmp->is_resolved_by($rpm);

=head1 DESCRIPTION

The RPM format involves 5-way comparisons.  A version requirement can
be one of: >, >=, ==, <= or <.  ie. "This module depends on foo-bar >=
1.2.4".

Doing that comparision dynamically can be tricky.  esmith::rpm::cmp
provides a pseudo-rpm which can figure out dependencies.  It's
primary reason for existence is the special cmp() method described below.

=head2 Methods

esmith::rpm::cmp provides a very limited subset of esmith::rpm.

=over 4

=item I<new>

  my $cmp = esmith::rpm::cmp->new($name, $version, $release, $sense);

Generates an esmith::rpm::cmp object representing a dependency on the
RPM $name and $version.  $sense is the RPM sense flag representing if
the dependency is >, >=, ==, <= or <.

If $release or $version are '' they will not be compared.

=cut

sub new {
    my($proto, $name, $version, $release, $sense) = @_;
    my($class) = ref $proto || $proto;

    warn "Release cannot be set if there is no version.  Ignoring release"
      if length $release && !length $version;

    my $self = bless { name    => $name,
                       version => $version,
                       release => $release,
                     };
    $self->_init_sense($sense);

    return $self;
}


=begin _private

=item I<_init_sense>

  $cmp->_init_sense($sense_flag);

Given an RPM $sense_flag it will initialize the $cmp object to do
comparisons against that sense.

=end _private

=begin testing

use RPM::Constants qw(:rpmsense);
require esmith::rpm::filename;
use Data::Dumper;

my %expect;
$expect{0}                                  = [[1,1,1], ''  ];
$expect{+RPMSENSE_GREATER}                  = [[0,0,1], '>' ];
$expect{RPMSENSE_EQUAL | RPMSENSE_GREATER}  = [[0,1,1], '>='];
$expect{+RPMSENSE_EQUAL}                    = [[0,1,0], '=='];
$expect{RPMSENSE_EQUAL | RPMSENSE_LESS}     = [[1,1,0], '<='];
$expect{+RPMSENSE_LESS}                     = [[1,0,0], '<' ];

my @vers = (0.5, 1.0, 1.5);
while( my($sense, $expect) = each %expect ) {
    my($eq, $str) = @$expect;
    my $cmp = esmith::rpm::cmp->new('foo', '1.0', '', $sense);
    is( $cmp->sense,        $sense );
    is( $cmp->sense_as_string, $str );
    is( $cmp->as_string,    "$str foo-1.0" );

    for my $idx (0..$#vers) {
        my $ver = $vers[$idx];
        $rpm = esmith::rpm::filename->new("foo-$ver-01.i386.rpm");
        is( $cmp->is_resolved_by($rpm), $eq->[$idx], Dumper($rpm).Dumper($cmp));
    }
}

=end testing

=cut


use RPM::Constants qw(:rpmsense);

my %sense2string;
$sense2string{0}                                  = '';
$sense2string{+RPMSENSE_GREATER}                  = '>';
$sense2string{RPMSENSE_EQUAL | RPMSENSE_GREATER}  = '>=';
$sense2string{+RPMSENSE_EQUAL}                    = '==';
$sense2string{RPMSENSE_EQUAL | RPMSENSE_LESS}     = '<=';
$sense2string{+RPMSENSE_LESS}                     = '<';

sub _init_sense {
    my($self, $flag) = @_;

    $self->{sense} = $flag;
    $self->{sense_string} = $sense2string{$flag};

    # RPM indicates "any version" with 000, not 111.  *sigh*
    $flag ||= RPMSENSE_LESS | RPMSENSE_EQUAL | RPMSENSE_GREATER;

    my @senses;
    push @senses, -1 if $flag & RPMSENSE_LESS;
    push @senses, 0  if $flag & RPMSENSE_EQUAL;
    push @senses, 1  if $flag & RPMSENSE_GREATER;

    $self->{_senses} = \@senses;

    return 1;
}


sub _num_equals {
    my($self, $num) = @_;

    return scalar grep { $num == $_ } @{$self->{_senses}};
}

=item I<name>

=item I<version>

=item I<release>

Returns the name, version and release we're comparing against.

=cut

sub name    { $_[0]->{name}    }
sub version { $_[0]->{version} }
sub release { $_[0]->{release} }

=begin _private

=item I<arch>

  my $arch = $cmp->arch;
  $cmp->arch($arch);

The architecture can be gotten and set via this method.  The $cmp
object must often change it's architecture to match that of the thing
being compared against.

=end _private

=cut

sub arch    {
    my($self) = shift;
    $self->{arch} = shift if @_;
    return $self->{arch};
}

=back

=head2 Additional Methods

=over 4

=item I<is_resolved_by>

  my $is_resolved = $cmp->is_resolved_by($rpm);

Returns true if the $rpm resolves the dependency $cmp represents.
This means having the right name and a version which compares
properly according to the sense of the $cmp.

=cut

sub is_resolved_by {
    my($self, $rpm) = @_;

    return 0 unless $self->name eq $rpm->name;

    # Make our arch match to fool cmp()
    $self->arch($rpm->arch);

    # temporarily set our version and release to match to
    # fool cmp().
    local $self->{version} = $rpm->version unless length $self->{version};
    local $self->{release} = $rpm->release unless length $self->{release};

    return $self->_num_equals($rpm->cmp($self));
}

=item I<as_string>

  my $cmp_as_string = $cmp->as_string;

A handy string representation of the comparison including the name,
version, release and sense.

    >= foo-bar-1.9-01

=cut

sub as_string {
    my($self) = shift;

    my $string = sprintf "%s %s", $self->sense_as_string,
      join('-', grep length, $self->name, $self->version, $self->release);
}

=item I<sense>

   my $sense_flag   = $cmp->sense;

Returns the raw RPM sense flag this $cmp represents.

=cut

sub sense { $_[0]->{sense} }


=item I<sense_as_string>

   my $sense_string = $cmp->sense_as_string;

Returns the comparision sense as a string.  ie. >, >=, ==, <=, or <.
No preference is indicated as an empty string.

=cut

sub sense_as_string { $_[0]->{sense_string} }

=back

=head1 AUTHOR

Mitel Networks Corporation

=cut

1;
