#----------------------------------------------------------------------
# Copyright 1999-2003 Mitel Networks Corporation
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
#----------------------------------------------------------------------

package esmith::rpm::file;

require 5.006;
use strict;
use warnings;
use Carp;

our $VERSION = sprintf '%d.%03d', q$Revision: 1.8 $ =~ /: (\d+).(\d+)/;

require esmith::rpm::header;
our @ISA = qw(esmith::rpm::header);

use RPM::Header;

=for testing
$CLASS = 'esmith::rpm::file';
use_ok($CLASS);


=head1 NAME

esmith::rpm::file - Class for manipulating RPM files.

=head1 SYNOPSIS

  use esmith::rpm::file;

  my $rpm  = 'esmith::rpm::file'->new($filename);

  my $name    = $rpm->name;
  my $version = $rpm->version;
  my $release = $rpm->release;
  my $arch    = $rpm->arch;

  my($name, $version, $release, $arch) = $rpm->tags;
  my $tag = $rpm->tag;

  $rpm1->cmp($rpm2);

  $rpm->reload;

  my $summary  = $rpm->summary;
  my $descrip  = $rpm->description;
  my @provides = $rpm->provides;

  my %requires         = $rpm->requires;
  my %unresolved_deps  = $rpm->unresolved_dependencies;

  esmith::rpm::file->install(@rpms);
  esmith::rpm::file->install_force(@rpms);


=head1 DESCRIPTION

Implements a subclass of esmith::rpm based on reading and parsing a
real RPM file.

This is a subclass of esmith::rpm::header.


=head2 Methods

The following methods of esmith::rpm are implemented and work based on
reading the headers of the given RPM file.

See esmith::rpm::header for details

=over 4

=item I<new>

  # due to a glitch in perl's parsing, you have to quote 
  # the class name to disambiguate from esmith::rpm::file()->new
  my $rpm = 'esmith::rpm::file'->new($rpm_filename);

Initializes a new object from the given $rpm_filename.

It can be either an RPM or a source RPM.

=for testing
$Rpm = $CLASS->new('10-e-smith-rpm/e-smith-rpm-0.0.1good-01.noarch.rpm');
isa_ok($Rpm, $CLASS);

=cut

sub new {
    my($proto, $rpm_filename) = @_;
    my($class) = ref $proto || $proto;

    my $header = RPM::Header->new($rpm_filename);
    return unless $header;

    my $self = $class->SUPER::new($header);
    $self->{filename} = $rpm_filename;
    return $self;
}


=item I<reload>

=cut

sub reload {
    my($self) = shift;
    %$self = $self->SUPER::new(RPM::Header->new($self->file));
}

=item I<file>

=cut

sub file { $_[0]->{filename} }

=item I<name>

=item I<version>

=item I<release>

=item I<arch>

=begin testing

is( $Rpm->file,     '10-e-smith-rpm/e-smith-rpm-0.0.1good-01.noarch.rpm' );
is( $Rpm->name,     'e-smith-rpm' );
is( $Rpm->version,  '0.0.1'       );
is( $Rpm->release,  '01'          );
is( $Rpm->arch,     'noarch'      );

=end testing

=item I<tag>

=item I<tags>

=for testing
is( join('',$Rpm->tags), join('', $Rpm->name, $Rpm->version,
                                  $Rpm->release, $Rpm->arch) );

=item I<cmp>

=item I<summary>

=for testing
is( $Rpm->summary, 'e-smith server and gateway - RPM manipulation utilities' );

=item I<description>

=for testing
is( $Rpm->description, 'e-smith modules & utilities for manipulating RPMs' );

=item I<provides>

=for testing
is_deeply( [$Rpm->provides], [qw(e-smith-rpm)] );

=item I<requires>

=begin testing

require esmith::rpm::filename;
my $rpm = 'esmith::rpm::file'->new(
                    '10-e-smith-rpm/e-smith-test-0.1.17-01.noarch.rpm'
                   );
my %requires = $rpm->requires;

my %expect = (
    'perl'    => { version => '',
                   cmp     => [1,1,1] },
    'perl-WWW-Automate' => { version => '0.20',
                             cmp     => [0,1,1], },
    'perl-Test-Simple'  => { version => '0.41',
                             cmp     => [0,1,1] },
    'perl-Test-Inline'  => { version => '0.14',
                             cmp     => [0,1,1] },
    'perl-Test-Harness-Straps' => { version => '0.09',
                                    cmp     => [0,1,0], },
    'e-smith-lib'       => { version => '1.9.14',
                             cmp     => [0,1,1], },
);

is_deeply([sort keys %requires], [sort keys %expect] );

my @version_adjust = (-0.01, 0, 0.01);
while( my($name, $expect) = each %expect ) {
    my $cmp = $requires{$name};
    is( $cmp->name,    $name,   "cmp $name");
    is( $cmp->version, $expect->{version}  );

    foreach my $idx (0..$#{$expect->{cmp}}) {
        my $adj_version = _ver_adj($cmp->version, $version_adjust[$idx]);
        my $rpm = esmith::rpm::filename->new("$name-$adj_version-01.noarch");
        is( $cmp->is_resolved_by($rpm), $expect->{cmp}[$idx], $adj_version );
    }
}


# Make sure an RPM with no dependencies does not blow up.
$rpm = esmith::rpm::header->new(RPM::Header->new(
                             '10-e-smith-rpm/qmailanalog-0.70-2.i386.rpm'
                            ));
eval { is_deeply( [$rpm->requires], [] ); };
is( $@, '' );


sub _ver_adj {
    my($ver, $adj) = @_;

    $ver ||= 5.6;
    return $ver unless $adj;
    $ver =~ s/(\d+).(\d+)$/"$1.".$2+$adj/e;
    return $ver;
}
                                           

=end testing

=item I<unresolved_dependencies>

=cut

=begin testing

$rpm = 'esmith::rpm::file'->new(
                   '10-e-smith-rpm/HasUnresolvedDeps-1.0.0-01.noarch.rpm'
                  );
my %unresolved = $rpm->unresolved_dependencies;

is_deeply( [sort keys %unresolved], [sort qw(NoUnresolvedDeps)] );

=end testing

=back


=head2 New Methods

These methods are added by esmith::rpm::file

=over 4

=item I<install>

=item I<install_force>

    my $ok = 'esmith::rpm::file'->install(@rpms);
    my $ok = 'esmith::rpm::file'->install_force(@rpms);

Installs the given set of @rpms.  install() will fail if there are any
problems.  install_force() will force the installation like the
--force flag.

Returns true if the install succeeded, false otherwise.

=cut

my $RPM_CMD = "/bin/rpm";
sub _install {
    my($class, $rpms, $options) = @_;
    $options = [] unless $options;

    my @rpm_files = map { $_->file } @$rpms;
    my @rpm_options = (qw( -U --quiet), @$options);

    my $res = system($RPM_CMD, @rpm_options, @rpm_files);
    return !$res;
}

sub install_force {
    my($class, @rpms) = @_;

    return $class->_install(\@rpms, [qw(--nodeps --force)]);
}

sub install {
    my($class, @rpms) = @_;

    return $class->_install(\@rpms);
}


=head1 AUTHOR

Mitel Networks Corporation

=cut

1;
