#----------------------------------------------------------------------
# Copyright 1999-2003 Mitel Networks Corporation
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
#----------------------------------------------------------------------

package esmith::rpm::filename;

require 5.006;
use strict;
use warnings;
use Carp;
use File::Basename;

our $VERSION = sprintf '%d.%03d', q$Revision: 1.17 $ =~ /: (\d+).(\d+)/;

require esmith::rpm;
our @ISA = qw(esmith::rpm);

# keep a list of the regex vars to export for convenience
use vars qw(@regex_vars);
BEGIN {
    @regex_vars = qw(
	RE_N			
	RE_V
	RE_R
	RE_ARCH
	RE_EXT
	RE_NV
	RE_NVR
	RE_VR
	RE_NVRARCH
	RE_FILENAME
    );
};
use vars (map {"\$$_"} @regex_vars);

# list the exported regex functions
$RE_N		= qr/([^\/]*?)/;
$RE_V		= qr/((?:\.?[\w\d\+]+)+)/;
$RE_R		= qr/((?:\.?[\w\d\+]+)+?)/;
$RE_ARCH	= qr/(i.86|noarch|sparc\d*|alpha|ppc|ia64|s390|athlon)/;
$RE_EXT		= qr/(rpm|srpm|src\.rpm)/;
$RE_NV 		= qr/$RE_N-$RE_V/o;
$RE_NVR 	= qr/$RE_N-$RE_V-$RE_R/o;
$RE_VR		= qr/$RE_V-$RE_R/o;
$RE_NVRARCH	= qr/$RE_NVR\.$RE_ARCH/o;
$RE_FILENAME 	= qr/$RE_NVR(?:\.$RE_ARCH)?\.$RE_EXT$/o;

sub import {
    my $proto = shift;
    my($class) = ref $proto || $proto;
    my($caller) = caller;

    foreach(@_) {
	# don't export regexes to subclasses unless they ask
	if (/^[-:]?regex$/)
	{
	    no strict 'refs';
	    foreach my $var (@regex_vars)
	    {
		*{"${caller}::$var"} = \${"${class}::$var"};
	    }
	}
    }
}

=head1 NAME

esmith::rpm::filename - RPM filename manipulation class

=head1 SYNOPSIS

  use esmith::rpm::filename;

  my $rpm = esmith::rpm::filename->new($rpm_filename);

  my $file    = $rpm->file;
  my $name    = $rpm->name;
  my $version = $rpm->version;
  my $release = $rpm->release;
  my $arch    = $rpm->arch;
  my $type    = $rpm->type;

  my($name, $version, $release, $arch) = $rpm->tags;
  my $tag = $rpm->tag;

  $rpm->reload;

  $rpm1->cmp($rpm2);

  -----

  use esmith::rpm::filename qw(regex);

  if ($name		=~ /^$RE_N$/) {}
  if ($version		=~ /^$RE_V$/) {}
  if ($release		=~ /^$RE_RELEASE$/) {}
  if ($arch 		=~ /^$RE_ARCH$/) {}
  if ($filename 	=~ /^$RE_FILENAME$/) {}
  if ($name_ver 	=~ /^$RE_NV$/) {}
  if ($name_ver_rel 	=~ /^$RE_NVR$/) {}
  if ($ver_rel 		=~ /^$RE_VR$/) {}
  if ($arch 		=~ /^$RE_ARCH$/) {}
  if ($nvr_arch		=~ /^$RE_NVRARCH$/) {}

=head1 DESCRIPTION

This class implements a subset of the functionality in esmith::rpm.
It's for working with I<just the filename> of an RPM.  The actual RPM
file need not exist and will not be used.

This is useful if you have large lists of RPM filenames.

=head2 Methods

The following methods of esmith::rpm are implemented and work based on
looking at the rpm's filename and B<NOT> by reading the spec file.

The actual RPM file need not exist.

See esmith::rpm for details.

=over 4

=item B<new>

The supplied $rpm_filename can be an rpm, srpm or just an rpm string
with no file ending.

    foo-1.0-01.noarch.rpm
    foo.1.0-01.noarch.srpm
    foo.1.0-01.noarch.src.rpm
    foo.1.0-01.noarch

It can be just the filename or a full path.

=cut

sub new {
    my($proto, $file) = @_;
    my($class) = ref $proto || $proto;

    my $self = bless { file => $file }, $class;
    unless( $self->_parse_filename ) {
        return;
    }

    return $self;
}

=begin _private

=item B<_parse_filename>

  $self->_parse_filename;

Parses an rpm filename into its components and stores it internally.

This:       'foo-3.9.1-01.noarch.rpm'
Expands to: 'foo', '3.9.1', '01', 'noarch'

To be called before any of name, version, release or arch.

=end _private

=cut

sub _parse_filename
{
    my($self) = shift;
    my $file = basename($self->file);
    my($name, $version, $release, $arch, $ext) =
     $file =~ m{^$RE_NVR(?:\.$RE_ARCH)?(?:\.$RE_EXT)?$}o or return;
    my $type = $self->_parse_ext($ext);

    @{$self}{qw(name version release arch type)} = 
	($name, $version, $release, $arch, $type);
    return 1;
}

sub _parse_ext {
    my($self, $ext) = @_;

    if( not defined $ext )
    {
        return 'string';
    }
    elsif( $ext eq 'rpm' ) {
        return 'rpm';
    }
    elsif( $ext eq 'srpm' || $ext eq 'src.rpm' ) {
        return 'srpm';
    }
    else {
        warn "Unknown rpm file extension: '$ext'";
    }

    return;
}


=item B<reload>

Since there's no file to re-read, this is a no-op.

=cut

sub reload { return 1 }

=item B<file>

=item B<name>

=item B<version>

=item B<release>

=item B<arch>

=cut

sub name    { $_[0]->{name} }
sub version { $_[0]->{version} }
sub release { $_[0]->{release} }
sub arch    { $_[0]->{arch} }

=item B<tag>

=item B<tags>

=item B<cmp>

It's safe to compare esmith::rpm objects to esmith::rpm::filename
objects.

=head2 New Methods

These are methods added by esmith::rpm::filename.

=over 4

=item B<type>

  my $type = $rpm->type;

The type of RPM filename given.  Can be one of 'rpm', 'srpm' or simply
'string' indicating the filename has no ending.

=cut

sub type { $_[0]->{type} }

=back

=head1 AUTHOR

Mitel Networks Corporation

=cut

1;
