#----------------------------------------------------------------------
# copyright (C) 2002 Mitel Networks Corporation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
#
# Technical support for this program is available from Mitel Networks
# Please visit our web site www.e-smith.com for details.
#----------------------------------------------------------------------

package esmith::rpm::filename;

require 5.006;
use strict;
use warnings;
use Carp;

our $VERSION = sprintf '%d.%03d', q$Revision: 1.1 $ =~ /: (\d+).(\d+)/;

require esmith::rpm;
our @ISA = qw(esmith::rpm);


=head1 NAME

esmith::rpm::filename - RPM filename manipulation class

=head1 SYNOPSIS

  use esmith::rpm::filename;

  my $rpm = esmith::rpm::filename->new($rpm_filename);

  my $file    = $rpm->file;
  my $name    = $rpm->name;
  my $version = $rpm->version;
  my $release = $rpm->release;
  my $arch    = $rpm->arch;

  esmith::rpm->cmp($rpm1, $rpm2);

=head1 DESCRIPTION

This class implements a subset of the functionality in esmith::rpm.
It's for working with I<just the filename> of an RPM.  The actual RPM
file need not exist and will not be used.

This is useful if you have large lists of RPM filenames.

=head2 Methods

The following methods of esmith::rpm are implemented and work based on
looking at the rpm's filename and B<NOT> by reading the spec file.

The actual RPM file need not exist.

See esmith::rpm for details.

=over 4

=item B<new>

=cut

sub new {
    my($proto, $file) = @_;
    my($class) = ref $proto || $proto;

    my $self = bless { file => $file }, $class;
    unless( $self->_parse_filename ) {
        carp "_parse_filename can't parse '$file'";
        return;
    }

    return $self;
}

=begin _private

=item B<_parse_filename>

  $self->_parse_filename;

Parses an rpm filename into its components and stores it internally.

This:       'foo-3.9.1-01.noarch.rpm'
Expands to: 'foo', '3.9.1', '01', 'noarch'

To be called before any of name, version, release or arch.

=end _private

=cut

sub _parse_filename
{
    my($self) = shift;
    my $file = $self->file;

    $file =~ /^(.*?)(-(\d[\w\d\.]*)-(\d[\w\d\.]*?)\.(\w[\w\d]*))?\.rpm$/ or
      return;

    @{$self}{qw(name version release arch)} = ($1, $3, $4, $5);
    return 1;
}


=item B<file>

=item B<name>

=item B<version>

=item B<release>

=item B<arch>

=cut

sub name    { $_[0]->{name} }
sub version { $_[0]->{version} }
sub release { $_[0]->{release} }
sub arch    { $_[0]->{arch} }

=item B<cmp>

It's safe to compare esmith::rpm objects to esmith::rpm::filename
objects.

=back

=head1 AUTHOR

Mitel Network Solutions, Inc.

=cut

1;
