#----------------------------------------------------------------------
# Copyright 1999-2003 Mitel Networks Corporation
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
#----------------------------------------------------------------------

package esmith::rpm::header;

require 5.006;
use strict;
use warnings;
use Carp;

our $VERSION = sprintf '%d.%03d', q$Revision: 1.14 $ =~ /: (\d+).(\d+)/;

require esmith::rpm;
our @ISA = qw(esmith::rpm);

use RPM::Header;

=for testing
$CLASS = 'esmith::rpm::header';
use_ok($CLASS);


=head1 NAME

esmith::rpm::header - Class for manipulating RPM::Header objects.

=head1 SYNOPSIS

  use esmith::rpm::header;

  my $rpm  = esmith::rpm::header->new($header);

  my $name    = $rpm->name;
  my $version = $rpm->version;
  my $release = $rpm->release;
  my $arch    = $rpm->arch;

  my($name, $version, $release, $arch) = $rpm->tags;
  my $tag = $rpm->tag;

  $rpm1->cmp($rpm2);

  $rpm->reload;

  my $serial     = $rpm->serial;
  my $summary    = $rpm->summary;
  my $descrip    = $rpm->description;
  my $packager   = $rpm->packager;
  my $buildtime  = $rpm->buildtime;
  my $buildhost  = $rpm->buildhost;
  my $sourcerpm  = $rpm->sourcerpm;

  my @provides = $rpm->provides;

  my %requires         = $rpm->requires;
  my %unresolved_deps  = $rpm->unresolved_dependencies;

  my @conflicts = $rpm->conflicts;                      *UNIMPLEMENTED*


=head1 DESCRIPTION

Implements a subclass of esmith::rpm based on reading and parsing a
RPM::Header object.

This is a subclass of esmith::rpm.


=head2 Methods

The following methods of esmith::rpm are implemented and work based on
reading given RPM::Header object.

See esmith::rpm for details

=over 4

=item I<new>

  my $rpm = esmith::rpm::header->new($header);

Initializes a new object from the given RPM::Header object.


=for testing
my $header = RPM::Header->new(
                '10-e-smith-rpm/e-smith-rpm-0.0.1good-01.noarch.rpm'
             );
$Rpm = $CLASS->new($header);
isa_ok($Rpm, $CLASS);

=cut

sub new {
    my($proto, $header) = @_;
    my($class) = ref $proto || $proto;

    return $class->_init($header);
}

# Dice up the RPM::Header into only the bits we need to save heaps
# of memory.
my @Headers = qw(name version release arch serial epoch
		 summary description
		 buildhost buildtime packager sourcerpm
		 provides
		 requirename requireflags requireversion
		 );

sub _init {
    my($class, $header) = @_;

    my %self = ( _header => { map { ($_,$header->{$_}) } @Headers } );

    return bless \%self, $class;
}
		    

=item I<reload>     ABSTRACT

Must be implemented by a subclass.

=item I<file>

=item I<name>

=item I<version>

=item I<release>

=item I<arch>

=begin testing

is( $Rpm->name,     'e-smith-rpm' );
is( $Rpm->version,  '0.0.1'       );
is( $Rpm->release,  '01'          );
is( $Rpm->arch,     'noarch'      );

=end testing

=cut

sub name    { $_[0]->{_header}{name} }
sub version { $_[0]->{_header}{version} }
sub release { $_[0]->{_header}{release} }
sub arch    { $_[0]->{_header}{arch} }

=item I<tag>

=begin testing

require esmith::rpm::filename;
my $new_rpm = esmith::rpm::filename->new($Rpm->tag);

is_deeply( [$new_rpm->tags], [$Rpm->tags] );

=end testing

=item I<tags>

=for testing
is( join('',$Rpm->tags), join('', $Rpm->name, $Rpm->version,
                                  $Rpm->release, $Rpm->arch) );

=item I<cmp>

esmith::rpm::headers sort slightly differently.  If either RPM has an
epoch (serial number) it will use that and ignore everything else but the
name.

=begin testing

my $new = esmith::rpm::header->new(
              RPM::Header->new('10-e-smith-rpm/with-serial-3.2.1-1.src.rpm')
          );
my $old = esmith::rpm::header->new(
              RPM::Header->new('10-e-smith-rpm/with-serial-4.5.6-1.src.rpm')
          );

# this one has no serial number
my $without = esmith::rpm::header->new(
              RPM::Header->new('10-e-smith-rpm/with-serial-4.5.6-2.src.rpm')
          );

my $rpm_filename = esmith::rpm::filename->new(
                          '10e-smith-rpm/with-serial-9.9.9-9.src.rpm'
          );

is( $new->serial, 45 );
is( $old->serial, 9  );
is( $without->serial, 0 );

is( $old->cmp($new), -1 );
is( $new->cmp($old), 1  );
is( $new->cmp($new), 0  );
is( $old->cmp($old), 0  );

# Comparing with RPMs that have no serial number.
is( $old->cmp($without), 1 );
is( $new->cmp($without), 1 );
is( $without->cmp($old), -1 );
is( $without->cmp($new), -1 );
is( $without->cmp($without), 0 );

# Comparing against non esmith::rpm::headers
is( $rpm_filename->cmp($new), 1 );
is( $new->cmp($rpm_filename), -1 );

=end testing

=cut

sub cmp {
    my($rpma, $rpmb) = @_;

    if( $rpmb->isa(__PACKAGE__) ) {
        return ($rpma->name cmp $rpmb->name)
                 ||
               ($rpma->serial <=> $rpmb->serial)
                 ||
               $rpma->SUPER::cmp($rpmb);
    }
    else {
        return $rpma->SUPER::cmp($rpmb);
    }
}

=back


=head2 New Methods

These methods are added by esmith::rpm::header.

=over 4

=item I<serial>

  my $serial = $rpm->serial;

Serial number of this RPM.  Not all RPMs have serial numbers, in which
case it will simply be 0. Also checks for the Epoch tag, which supercedes
Serial.

=item I<summary>

  my $summary = $rpm->summary;

Short description of the $rpm.

=for testing
is( $Rpm->summary, 'e-smith server and gateway - RPM manipulation utilities' );

=item I<description>

  my $description = $rpm->description;

Full description of the $rpm.

=for testing
is( $Rpm->description, 'e-smith modules & utilities for manipulating RPMs' );

=cut

sub summary     { $_[0]->{_header}{summary} }
sub description { $_[0]->{_header}{description} }
sub serial      { $_[0]->{_header}{serial} || $_[0]->{_header}{epoch} || 0 }

=item I<buildhost>

  my $host = $rpm->buildhost;

Host upon which this RPM was built.

=for testing
is( $Rpm->buildhost, 'lamington.nssg.mitel.com' );

=item I<buildtime>

  my $time = $rpm->buildtime;

Unix epoch time when this RPM was built.

=for testing
is( $Rpm->buildtime, '1017944909' );

=item I<packager>

  my $packager = $rpm->packager;

Identity of the person who packaged this RPM.

=for testing
is( $Rpm->packager, 'e-smith developers <bugs@e-smith.com>' );

=item I<sourcerpm>

  my $sourcerpm = $rpm->sourcerpm;

Source RPM from which this binary RPM was built.

=for testing
is( $Rpm->sourcerpm, 'e-smith-rpm-0.0.1-01.src.rpm' );

=cut

sub buildhost { $_[0]->{_header}{buildhost} }
sub buildtime { $_[0]->{_header}{buildtime} }
sub packager  { $_[0]->{_header}{packager}  }
sub sourcerpm  { $_[0]->{_header}{sourcerpm}  }

=item I<provides>

  my @provides = $rpm->provides;

Returns a list of what this $rpm provides.

=for testing
is_deeply( [$Rpm->provides], [qw(e-smith-rpm)] );

=cut

sub provides {
    my($self) = shift;

    return @{$self->{_header}{provides}};
}

=item I<requires>

  my %requires = $rpm->requires;

Returns what other RPMs this $rpm depends on.

  $requires->{$name} = $cmp

$cmp is an esmith::rpm::cmp object representing the complicated
version comparison necessary to determine if a given RPM resolves the
dependency.

    printf "%s satsfies %s's dependency\n", $other_rpm->tag, $rpm->tag
        if $cmp->is_resolved_by($other_rpm);


=begin testing

my $header = RPM::Header->new(
                  '10-e-smith-rpm/e-smith-test-0.1.17-01.noarch.rpm'
             );
my $rpm = esmith::rpm::header->new($header);

my %requires = $rpm->requires;

my %expect = (
    'perl'    => { version => '',
                   cmp     => [1,1,1] },
    'perl-WWW-Automate' => { version => '0.20',
                             cmp     => [0,1,1], },
    'perl-Test-Simple'  => { version => '0.41',
                             cmp     => [0,1,1] },
    'perl-Test-Inline'  => { version => '0.14',
                             cmp     => [0,1,1] },
    'perl-Test-Harness-Straps' => { version => '0.09',
                                    cmp     => [0,1,0], },
    'e-smith-lib'       => { version => '1.9.14',
                             cmp     => [0,1,1], },
);

is_deeply([sort keys %requires], [sort keys %expect] );

my @version_adjust = (-0.01, 0, 0.01);
while( my($name, $expect) = each %expect ) {
    my $cmp = $requires{$name};
    is( $cmp->name,    $name,   "cmp $name");
    is( $cmp->version, $expect->{version}  );

    foreach my $idx (0..$#{$expect->{cmp}}) {
        my $adj_version = _ver_adj($cmp->version, $version_adjust[$idx]);
        my $rpm = esmith::rpm::filename->new("$name-$adj_version-01.noarch");
        is( $cmp->is_resolved_by($rpm), $expect->{cmp}[$idx], $adj_version );
    }
}


# Make sure an RPM with no dependencies does not blow up.
$rpm = esmith::rpm::header->new(RPM::Header->new(
                             '10-e-smith-rpm/qmailanalog-0.70-2.i386.rpm'
                            ));
eval { is_deeply( [$rpm->requires], [] ); };
is( $@, '' );


sub _ver_adj {
    my($ver, $adj) = @_;

    $ver ||= 5.6;
    return $ver unless $adj;
    $ver =~ s/(\d+).(\d+)$/"$1.".$2+$adj/e;
    return $ver;
}


=end testing

=cut  

use RPM::Constants qw(:rpmsense);
use esmith::rpm::cmp;
sub requires {
    my($self) = shift;

    # No dependencies.
    return () unless $self->{_header}{requirename};

    my %requires = ();
    my @names = @{$self->{_header}{requirename}};
    my @flags = @{$self->{_header}{requireflags}};
    my @vers  = @{$self->{_header}{requireversion}};
    foreach my $idx (0..$#names) {
        my $name = $names[$idx];
        my $flag = $flags[$idx];
        my $ver  = $vers[$idx] || '';
        my $release;
        if( $ver =~ s/-(.*)// ) {
            $release = $1;
        }
        else {
            $release = '';
        }

        # Something other than an RPM.
        next if $flag & ~RPMSENSE_SENSEMASK;

        my $sense = $flag & RPMSENSE_SENSEMASK;
        $requires{$name} = 'esmith::rpm::cmp'->new($name, $ver, $release, 
                                                   $sense);
    }

    return %requires;
}

=item I<unresolved_dependencies>

  my %unresolved = $rpm->unresolved_dependencies;
  my %unresolved = $rpm->unresolved_dependencies(@rpms);

Like requires(), only it will return a hash of only those dependencies
which have not yet been resolved (ie. installed).  If extra @rpms are
given it will use those in addition to those installed to determine
dependency resolution.

=begin testing

my $header = RPM::Header->new(
                   '10-e-smith-rpm/HasUnresolvedDeps-1.0.0-01.noarch.rpm'
                  );
my $rpm = esmith::rpm::header->new($header);
my %unresolved = $rpm->unresolved_dependencies;

is_deeply( [sort keys %unresolved], [sort qw(NoUnresolvedDeps)] );

=end testing

=cut

sub unresolved_dependencies {
    my($self, @extra_rpms) = @_;

    my %extra_rpms = ();
    foreach my $rpm (@extra_rpms) {
        my $name = $rpm->name;
        push @{$extra_rpms{$name}}, $rpm;
    }

    my %unresolved = ();
    my %requires = $self->requires;
    require esmith::rpm::installed;
    while( my($name, $cmp) = each %requires ) 
    {
        my @checks = esmith::rpm::installed->new($name) || ();
        push @checks, @{$extra_rpms{$name}} if $extra_rpms{$name};

        my $resolved = 0;
        CHECK: foreach my $check (@checks) {
            if( $cmp->is_resolved_by($check) ) {
                $resolved = 1;
                last CHECK;
            }
        }

        $unresolved{$name} = $cmp unless $resolved;
    }

    return %unresolved;
}



=back

=head1 AUTHOR

Mitel Networks Corporation

=cut

1;
