#----------------------------------------------------------------------
# Copyright 1999-2003 Mitel Networks Corporation
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
#----------------------------------------------------------------------

package esmith::rpm::installed;

require 5.006;
use strict;
use warnings;
use Carp;

our $VERSION = sprintf '%d.%03d', q$Revision: 1.8 $ =~ /: (\d+).(\d+)/;

require esmith::rpm::header;
our @ISA = qw(esmith::rpm::header);

require RPM::Database;

=for testing
$CLASS = 'esmith::rpm::installed';
use_ok($CLASS);


=head1 NAME

esmith::rpm::installed - Class for manipulating installed RPMs.

=head1 SYNOPSIS

  use esmith::rpm::installed;

  my @all_rpms = esmith::rpm::installed->installed;
  my @rpms = esmith::rpm::installed->installed($name);

  my $rpm  = esmith::rpm::installed->new($name);

  my $name    = $rpm->name;
  my $version = $rpm->version;
  my $release = $rpm->release;
  my $arch    = $rpm->arch;

  my($name, $version, $release, $arch) = $rpm->tags;
  my $tag = $rpm->tag;

  $rpm1->cmp($rpm2);

  $rpm->reload;

  my $summary  = $rpm->summary;
  my $descrip  = $rpm->description;
  my @provides = $rpm->provides;

  my %requires         = $rpm->requires;
  my %unresolved_deps  = $rpm->unresolved_dependencies;
  my @requires         = $rpm->what_requires;

  esmith::rpm::installed->uninstall(@rpms);
  esmith::rpm::installed->uninstall_force(@rpms);


=head1 DESCRIPTION

Implements a subclass of esmith::rpm::header based on reading the system's
database of installed RPMs.

=head2 Methods

The following methods of esmith::rpm are implemented and work based on
reading the headers of the given RPM file.

See esmith::rpm for details

=over 4

=item I<new>

  my $rpm = esmith::rpm::file->new($rpm_name);

Initializes a new object from the given $rpm_name.


=for testing
# we depend on rpm-perl, so it has to be installed
$Rpm = $CLASS->new('rpm-perl');
isa_ok($Rpm, $CLASS);

=cut

sub new {
    my($proto, $rpm_name) = @_;
    my($class) = ref $proto || $proto;

    my $DB = RPM::Database->new;
    my $self = bless {DB => $DB}, $class;
    return unless $self->_read_header($rpm_name);
    return $self;
}


sub _read_header {
    my($self, $name) = @_;

    $self->{_header} = $self->{DB}->{$name};
    return defined $self->{_header};
}


=item I<reload>

=item I<file>

=item I<name>

=item I<version>

=item I<release>

=item I<arch>

=item I<tag>

=item I<tags>

=for testing
is( join('',$Rpm->tags), join('', $Rpm->name, $Rpm->version,
                                  $Rpm->release, $Rpm->arch) );

=item I<cmp>

=item I<summary>

=item I<description>

=item I<provides>

=item I<requires>

=begin testing

require esmith::rpm::filename;
my %requires = $Rpm->requires;

my %expect = (
#    'perl'    => { version => '5.6.0',
#                   release => '9',
#                   cmp     => [1,1,1] },
);

#is_deeply([sort keys %requires], [sort keys %expect] );

my @version_adjust = (-0.01, 0, 0.01);
while( my($name, $expect) = each %expect ) {
    my $cmp = $requires{$name};
    is( $cmp->name,    $name );
    is( $cmp->version, $expect->{version} );
    is( $cmp->release, $expect->{release} );

    foreach my $idx (0..$#{$expect->{cmp}}) {
        my $adj_ver = $cmp->version + $version_adjust[$idx];
        my $rel = $expect->{release};

        my $rpm = esmith::rpm::filename->new("$name-$adj_ver-$rel.noarch");
        is( $cmp->is_resolved_by($rpm), $expect->{cmp}[$idx], $adj_ver );
    }
}
                                           

=end testing

=item I<unresolved_dependencies>

=head2 New Methods

These methods are added by esmith::rpm::installed.

=over 4

=item I<what_requires>

  my @requires = $rpm->what_requires;

Returns a list of esmith::rpm::installed RPMs which depend on $rpm.

=begin testing

# We know e-smith-test requires perl-Test-Simple and we require
# e-smith-test, so it should be installed.
my $rpm = esmith::rpm::installed->new('perl-Test-Simple');
my @requires = $rpm->what_requires;

ok( grep { $_->name eq 'e-smith-test' } @requires );


=end testing

=cut

sub what_requires {
    my($self) = shift;

    return map { esmith::rpm::installed->new($_->{name}) }
           $self->{DB}->find_what_requires($self->name);
}

=back


=head2 New Methods

These methods are added by esmith::rpm::installed.

=over 4

=item I<uninstall>

=item I<uninstall_force>

    my $ok = esmith::rpm::installed->uninstall(@rpms);
    my $ok = esmith::rpm::installed->uninstall_force(@rpms);

Uninstalls the given set of @rpms.  uninstall() will fail if there are
any problems.  uninstall_force() will force the uninstallation like
the --force flag.

Returns true if the uninstall succeeded, false otherwise.

=cut

my $RPM_CMD = "/bin/rpm";
sub _uninstall {
    my($class, $rpms, $options) = @_;
    $options = [] unless $options;

    my @rpm_names = map { $_->name } @$rpms;
    my @rpm_options = (qw( -e --quiet), @$options);

    my $res = system($RPM_CMD, @rpm_options, @rpm_names);
    return !$res;
}

sub uninstall_force {
    my($class, @rpms) = @_;

    return $class->_uninstall(\@rpms, [qw(--nodeps --force)]);
}

sub uninstall {
    my($class, @rpms) = @_;

    return $class->_uninstall(\@rpms);
}


=head1 AUTHOR

Mitel Networks Corporation

=cut

1;

