#----------------------------------------------------------------------
# Copyright 1999-2003 Mitel Networks Corporation
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
#----------------------------------------------------------------------


package esmith::rpm::macros;

=head1 NAME

  esmith::rpm::macros - manipulate simple rpm macros

=head1 SYNOPSIS

  my $m = esmith::rpm::macros->new;

  $m->read_default_macros;
  print $m->expand_macro("_tmppath");
     /var/tmp

  my $val = $m->get_macro("_tmppath");
  print $m->expand_string($val);
     /var/tmp

  $m->add_macro("my_path", "%{_tmppath}/my/path");
  print $m->expand_macro("my_path");
    /var/tmp/my/path

  $m->add_macro("_tmppath", "/tmp");
  print $m->expand_macro("my_path");
    /tmp/my/path

  $m->del_macro("_tmppath");
  print $m->expand_macro("my_path");
    %{_tmppath}/my/path

  $m->read_from_macro_file("/path/to/macro/file");
  $m->read_from_spec_file("/path/to/spec/file");
  $m->print_all;
    ...
  $m->print_all_noexpand;
    ...

=head1 BUGS

  Only simple macros in the form %{\w+} are expandable.  
  Parameterized macros and macros that use conditional/negation modifiers are
  not expandable.  
  Shell commands of the form %(.+) are not expandable.

=cut

sub new
{
    my $proto = shift;
    my $class = ref($proto) || $proto;
    return bless [], $class;
}

sub read_default_macros
{
    my $self = shift;

    foreach $file ('/usr/lib/rpm/macros', 
		   '/etc/rpm/macros', 
		   "$ENV{HOME}/.rpmmacros")
    {
	$self->read_from_macro_file($file) if -r $file;
    }
}

sub read_from_macro_file
{
    my $self = shift;
    my $file = shift;

    open (MACROS, "<$file")
	or die "Can't open macro file $file for reading";

    my $cur = "";
    my $line;
    while ($line = <MACROS>)
    {
	chomp $line;
	# skip leading hash
	next if $line =~ /^\s*(?:#|\s*$)/;
	$cur .= $line;
	# skip trailing hash
	$cur =~ s/([^\\])#.*$/$1/;
	# concatenate lines ending with \
	next if $line =~ s/\\$//;
	push @$self, $cur;
	$cur = "";
    }
    close MACROS;
}

sub read_from_spec_file
{
    my $self = shift;
    my $file = shift;

    open (SPEC, "<$file")
	or die "Can't open spec file $file for reading";

    while ($line = <SPEC>)
    {
	chomp $line;
	if ($line =~ /^\s*%define\s+(.*)$/)
	{
	    push @$self, "\%$1";
	}
    }
    close SPEC;
}

sub add_macro
{
    my $self = shift;
    my $key = shift;
    my $val = shift;

    push @$self, "\%$key  $val";
}

sub del_macro
{
    my $self = shift;
    my $key = shift;
    @$self = grep {!/^$key\s/} @$self;
}

sub get_macro
{
    my $self = shift;
    my $macro = shift;
    my ($found) = (grep {m/^%$macro\s/} @$self)[-1];
    return $found ? (split(/\s+/, $found, 2))[1] : undef;
}

sub expand_macro
{
    my $self = shift;
    my $macro = shift;
    local $SIG{__WARN__} = sub 
    {
	$_[0] =~ /^Deep recursion/ 
	    && die "ERROR: Infinate loop in define of rpm macro '$macro'\n";
	warn @_;
    };
    return undef unless defined $macro;
    my $val = $self->get_macro($macro);
    if ($macro =~ /^PACKAGE_(.*)$/ && !$val) 
    {
	return $self->expand_macro(lc($1));
    }
    # return the original string if we can't expand it
    return defined($val) ? $self->expand_string($val) : "%{$macro}";
}

sub expand_string
{
    my $self = shift;
    my $string = shift;
    return undef unless defined $string;

    $string =~ s{(?<!%)%\{(\w+?[^\\])\}}
	        { $1 eq '!?nil' ? '' : $self->expand_macro($1) }eg;
    return $string;
}


sub print_all
{
    my $self = shift;

    foreach my $key ( map {(split(/\s+/, $_, 2))[0]} @$self)
    {
	$key =~ s/^%//;
	print "$key     " .$self->expand_macro($key). "\n";
    }
}

sub print_all_noexpand
{
    my $self = shift;

    foreach my $line (@$self)
    {
	print "$line\n";
    }
}

1;
