#----------------------------------------------------------------------
# Copyright 1999-2003 Mitel Networks Corporation
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
#----------------------------------------------------------------------

package esmith::rpm::spec;

require 5.006;
use strict;
use warnings;
use Carp;

use constant TRUE => 1;
use constant FALSE => 0;

our $VERSION = sprintf '%d.%03d', q$Revision: 1.13 $ =~ /: (\d+).(\d+)/;

require esmith::rpm;
require esmith::rpm::macros;

our @ISA = qw(esmith::rpm);


=head1 NAME

esmith::rpm::spec - RPM spec file manipulation class

=head1 SYNOPSIS

  use esmith::rpm::spec;

  my $rpm = esmith::rpm::spec->new($specfile);

  my $file    = $rpm->file;
  my $name    = $rpm->name;
  my $version = $rpm->version;
  my $release = $rpm->release;
  my $arch    = $rpm->arch;

  my($name, $version, $release, $arch) = $rpm->tags;
  my $tag = $rpm->tag;

  $rpm->reload;

  my $spec    = $rpm->spec;

  $rpm1->cmp($rpm2);

=head1 DESCRIPTION

Implements a subset of esmith::rpm based on reading and parsing an RPM
spec file.

This is a subclass of esmith::rpm.

=head2 Methods

The following methods of esmith::rpm are implemented and work based on
reading the given spec file.

See esmith::rpm for details.

=over 4

=item B<new>

  my $rpm = esmith::rpm::spec->new($specfile);

Initializes a new object from a given $specfile.

=cut

sub new {
    my($proto, $specfile) = @_;
    my($class) = ref $proto || $proto;

    my $self = bless { spec => $specfile }, $class;
    $self->_read_specfile;

    return $self;
}


sub _read_specfile {
	my($self) = shift;
	my %specials = map { $_ => 1 } (qw(Name Version Release));

	# catch errors, report file name
	local $SIG{__DIE__} = sub 
	{
	    die $_[0]." in spec file ".$self->spec;
	};
	$self->{macros} = esmith::rpm::macros->new;
	my $macros = $self->{macros};

	$macros->read_default_macros;

	my $reading = FALSE;
	open(SPEC, $self->spec) || die $!;
	while( my $line = <SPEC> ) {
		chomp $line;

		# Skip blank lines above the specfile header.
		if (! $reading) {
			if ($line !~ /^\s*$/) {
				$reading = TRUE;
			}
			else {
				next;
			}
		}

		last if $line =~ /^%(?!define\s)/;   # end of headers
		next if $line =~ /^\s*#/;   # comment
		next if $line =~ /^\s*$/;   # blank line

		if( $line =~ /^(\w+):\s*(.*)/ )
		{
			my($key, $val) = ($1, $2);
			if (defined $specials{$key} &&
			    !defined $macros->get_macro(lc($key)))
			{
				# Define implicit macros when we see the assoc.
				# header
				$macros->add_macro($key,$val);
			}
			$val = $macros->expand_string($val);
			$self->{_spec}{headers}{$key} = $val;
		}
		elsif( $line =~ /^%define\s*(\S+)\s*(.*)/ )
		{
			my($key, $val) = ($1, $2);
			$macros->add_macro($key, $val);
		}
		else {
			warn sprintf "I don't understand line %d of %s\n%s\n",
			$., $self->spec, $line;
		}
	}
	close SPEC;
}

=item B<reload>

=cut

sub reload {
    my($self) = shift;

    %$self = %{$self->new($self->spec)};
    return 1;
}

=item B<file>

Returns the rpm filename which this spec file would generate.

=cut

sub file {
    my($self) = shift;
    return sprintf "%s-%s-%s.%s.rpm", map { $self->$_() }
              qw(name version release arch);
}

=item B<headers>

=cut

sub headers
{
    my($self) = shift;
	if (exists $self->{_spec}{headers}) {
		return (%{$self->{_spec}{headers}});
	}
	else {
		my $msg=<<EOF;
Invalid rpm header. No recognized name: value pairs found.
EOF
		die $msg;
	}
}

=item B<name>

=item B<version>

=item B<release>

=item B<arch>

=cut

sub name    { $_[0]->{_spec}{headers}{Name} }
sub version { $_[0]->{_spec}{headers}{Version} }
sub release { $_[0]->{_spec}{headers}{Release} }
sub arch    { $_[0]->_target_cpu }

=item B<_target_cpu>

Fetch the _target_cpu macro for the spec file.

=cut

sub _target_cpu
{
    $_[0]->{_spec}->{headers}{BuildArchitectures} ||
    $_[0]->{_spec}->{headers}{BuildArch} ||
    $_[0]->SUPER::_target_cpu;
}

=item B<tag>

=item B<tags>

=item B<cmp>

=back


=head2 New Methods

These are methods added by esmith::rpm::spec.

=over 4

=item B<spec>

  my $spec = $rpm->spec;

The spec file the $rpm is reading from.

=cut

sub spec { $_[0]->{spec} }

=back

=head1 AUTHOR

Mitel Networks Corporation

=cut

1;
